#include<stdlib.h>
#include <string.h>

#include "kos32sys1.h"
#include "ChildWin.h"

char proc_info_child[1024];

#define FONT_W 8
#define FONT_H 14
char*   linebuffer[1000];  // array of strings displayed area
int     lb_max; // sizeof ^ fit in window
int     lb_cur; // no of used lines


void reformat_linebuffer(childparam_t *child)
{
    char *ptr, *lptr;
    int cur_line, linewidth;
    lb_max = (child->hight - get_skin_height() - 5) / FONT_H; // 25 titlebar, 5 borders
    linewidth = (child->width - 10) / FONT_W;

    lb_cur = 0;
    for (ptr = child->start_draw, cur_line = 0; *ptr && cur_line < lb_max; cur_line++)
    {
        linebuffer[cur_line] = realloc(linebuffer[cur_line], linewidth + 1); linebuffer[cur_line][linewidth] = '\0';
        for (lptr = linebuffer[cur_line]; *ptr; ptr++)
        {
            if(*ptr == '\r' || *ptr == '\n')
            {
                if (*ptr == '\r' && *(ptr + 1) == '\n') ptr++;
                ptr++;
                break;
            }
            else
                *lptr++ = *ptr;
            if (lptr - linebuffer[cur_line] >= linewidth)
            {
                ptr++;
                break;
            }
        }
        *lptr = '\0';
        lb_cur = cur_line + 1;
    }
    debug_board_printf("linebuffer max, cur = %d, %d\n", lb_max, lb_cur);

}

void draw_window_child(childparam_t *child)
{
    int win_hight, win_width, i, pos_y = get_skin_height();

    // start redraw
    begin_draw();
	// define&draw window
	sys_create_window(child->left, child->top, child->width, child->hight, child->title, /*sys_color_table.work_area*/0xFFFFFF, 0x13);

    get_proc_info(proc_info_child);
    win_width = *(int*)(proc_info_child + 0x3E); // client, 2A windows
    win_hight = *(int*)(proc_info_child + 0x42); // client, 2E windows

    if(win_width != child->width || win_hight != child->hight) // resize
    {
        child->width = win_width;
        child->hight = win_hight;
        reformat_linebuffer(child);
    }
//    draw_text_sys("center", (win_width - 8 * 6) /2, (win_hight - 12 * 1) /2 , 0, 0x90000000 | sys_color_table.work_text);  //0x80000000 asciiz

/*
    // display statusbar
    draw_bar(6, win_hight - 17, win_width - 11, 12, 0x80000000 | sys_color_table.work_area);  //0x80000000 gradient
    draw_text_sys(statusbar, 10, win_hight - 15, 0, 0x80000000 | sys_color_table.work_text);
*/
    for (i = 0; i < lb_cur; i++)
    {
        draw_text_sys(linebuffer[i], 5, pos_y, 0, 0x90000000 /*| sys_color_table.work_text*/);
        pos_y += FONT_H;
    }

    // end redraw
    end_draw();
}


int child_window_create(childparam_t *child)
// return value as sysfn 51
{
    int rc = -2;
    char    *stack_top;

    if(!child) return rc;

    if(child->flags & CENTERSCREEN)
    {
        int xy = GetScreenSize(), maxx, maxy;// get screen size
        maxx = xy >> 16;
        maxy = xy & 0xFFFF;
        child->left = (maxx - child->width) / 2;
        child->top = (maxy - child->hight) / 2;
    }
    if(child->flags & CENTERPARENT)
    {
        if(child->width > child->p_width)
            child->left = child->p_left - (child->width - child->p_width) / 2;
        else
            child->left = child->p_left + (child->p_width - child->width) / 2;
        if(child->hight > child->p_hight)
            child->top = child->p_top - (child->hight - child->p_hight) / 2;
        else
            child->top = child->p_top + (child->p_hight - child->hight) / 2;
    }
    if (child->left < 0) child->left = 0;
    if (child->top < 0) child->top = 0;

    if(child->flags & MODAL)
        child->modal_state = 1;

    child->start_draw = child->param;
    reformat_linebuffer(child);

    stack_top = child->stack_child + sizeof(child->stack_child) - 8;
    *(void**)(stack_top + 4) = child;

    rc = start_thread((thread_proc)child_main, stack_top);
    child->tid = rc;
    child->slot = get_thread_slot(rc);
    return rc;

}

void child_main(childparam_t* child)
{
    int gui_event = KOLIBRI_EVENT_REDRAW;
    uint32_t pressed_button = 0, mouse_button;
    pos_t mouse_pos;

    debug_board_printf("running dialog[%X](%X)\n", child, child->param);

    //set_event_mask(0xC0000027); // mouse events only when focused window and mouse inside

    do  /* Start of main activity loop */
    {
        switch(gui_event)
        {
        case KOLIBRI_EVENT_NONE:
			break;
        case KOLIBRI_EVENT_REDRAW:
            draw_window_child(child);
			break;
        case KOLIBRI_EVENT_KEY:
            // scroll
			break;
        case KOLIBRI_EVENT_BUTTON:
            pressed_button = get_os_button();
            switch (pressed_button)
            {
              case BTN_QUIT:
                child->modal_state = 0;
                child->result = 0;  // closed by X
                kos_exit();
                break;
            }
            break;
        case KOLIBRI_EVENT_MOUSE:
            mouse_pos = get_mouse_pos(POS_WINDOW); // window relative
            mouse_button = get_mouse_eventstate();
            // ignore
            break;
        }
        gui_event = get_os_event();
    } while(1) ; /* End of main activity loop */

  return;
}
