/*
    Small clipboard viewer program for KolibriOS

    Free for all

    Initially written by Siemargl, 2016


    ToDo

    tedit - gpf - suspect overflow in draw_text_sys()
    boxlib editbox error - copies not only marked text but all line
    eolite - clipboard formar is nonstandard - 10 bytes header only

    big program size - linker add much of libc
    add encoding conversion to 866
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "kos32sys1.h"
#include "ChildWin.h"

struct kolibri_system_colors sys_color_table;

char statusbar[255];
char proc_info[1024];
char clip_line[14096];


void draw_window()
{
    int win_hight, win_width, i, pos_y = get_skin_height() + 36;  // 60 == 24+36

    // start redraw
    begin_draw();
	// define&draw window
	sys_create_window(10, 40, 600, 400, "Clipboard Viewer", /*sys_color_table.work_area*/0xFFFFFF, 0x13);

    get_proc_info(proc_info);
    win_width = *(int*)(proc_info + 0x3E); // client, 2A windows
    win_hight = *(int*)(proc_info + 0x42); // client, 2E windows

	define_button((10 << 16) + 40, (30 << 16) + 20, BTN_POP, sys_color_table.work_button);
    draw_text_sys("POP", 15, 34, 0, 0x90000000 | sys_color_table.work_button_text);  //0x80000000 asciiz

	define_button((65 << 16) + 60, (30 << 16) + 20, BTN_UNLOCK, sys_color_table.work_button);
    draw_text_sys("UNLOCK", 70, 34, 0, 0x90000000 | sys_color_table.work_button_text);

    // display statusbar
    draw_bar(6, win_hight - 17, win_width - 11, 12, 0x80000000 | sys_color_table.work_area);  //0x80000000 gradient
    draw_text_sys(statusbar, 10, win_hight - 15, 0, 0x80000000 | sys_color_table.work_text);

    // display clipboard
    for (i = kol_clip_num(); i > 0; i--)
    {
        *clip_line = 0;
        char *clip_item = kol_clip_get(i - 1);
        uint32_t nbytes, fmt, extra = 0;
        if((int)clip_item == 1 || (int)clip_item == -1) break;

        nbytes = *(int32_t*)clip_item;
        if(nbytes < 10) break;

        fmt = *(int32_t*)(clip_item + 4);
        switch (fmt)
        {
        case 0:  // TEXT
            extra = *(int32_t*)(clip_item + 8);  // encoding 0 = UTF, 1 = 0866, 2 = 1251
            tiny_snprintf (clip_line, sizeof clip_line, "TEXT[%d]%s", nbytes, clip_item + 12);  // add encoding conversion
            clip_line[sizeof clip_line - 1] = '\0';
            break;
        case 1:  // BMP
            tiny_snprintf (clip_line, sizeof clip_line, "BMP[%d]<<IMAGE>>", nbytes);
            break;
        case 2:  // RAW
            tiny_snprintf (clip_line, sizeof clip_line, "RAW[%d]<<BINARY>>", nbytes);
            break;
        case 3:  // EOLITE file list
            extra = *(uint16_t*)(clip_item + 8);  // unusual, only 16bit value, n items
            {
                char *fname = clip_item + 10;
                int flen, clip_line_len, item;
                tiny_snprintf (clip_line, sizeof clip_line, "FLIST[%d]", extra);
                clip_line_len = strlen(clip_line);
                for(item = 0; item < extra; item++)
                {
                    flen = strlen(fname);
                    if(clip_line_len + flen >= sizeof clip_line - 1) break;
//                    strcat(clip_line, fname);
//                    strcat(clip_line, ";");
                    strcat(clip_line + clip_line_len, fname);
                    clip_line[clip_line_len + flen] = ';';
                    clip_line[clip_line_len + flen + 1] = '\0';
//                    strcat(clip_line + clip_line + flen, ";");
                    clip_line_len += flen + 1;
                    fname += 4096;
                }
            }
            break;
        default:  // unknown
            tiny_snprintf (clip_line, sizeof clip_line, "UNKNOWN[%d]<<BINARY>>", nbytes);
            break;
        }

        clip_line[(win_width - 10 - 5) / FONT_W + 1] = '\0'; // clip text size, seems to big lines crashing OS, and form len by window size
//        draw_number_sys(nbytes, 5, pos_y, 6, 0x10000000); 8x12 font
        draw_text_sys(clip_line, 5, pos_y, 0, 0x90000000 /*| sys_color_table.work_text*/);
        pos_y += FONT_H;

        user_free(clip_item);

        if(pos_y + 29 > win_hight) break; // 12 font + 12 statusbar + 5 border
    }

    // end redraw
    end_draw();
}

childparam_t dlg;
void show_clipslot_modal(int clip_slot);

int main()
{
    int gui_event = KOLIBRI_EVENT_REDRAW;
    uint32_t pressed_button = 0, mouse_button;
    int prev_clip_num = -10, last_clip_num;
    pos_t   mouse_pos;
    strcpy(statusbar, "ClipView running...Double click on TEXT for details");

    get_system_colors(&sys_color_table);
    set_event_mask(0xC0000027); // mouse events only when focused window and mouse inside

    //init dialog
    dlg.title = "Contens";
    dlg.flags |= MODAL | CENTERPARENT;
    dlg.width = 500;
    dlg.hight = 350;

    dlg.p_top = 40;
    dlg.p_left = 10;
    dlg.p_width = 600;
    dlg.p_hight = 400;

    do  /* Start of main activity loop */
    {
        if(gui_event == KOLIBRI_EVENT_REDRAW)
            draw_window();

        // imitate modal dialog
        if (dlg.modal_state)
        {
            if (gui_event != KOLIBRI_EVENT_REDRAW && gui_event != KOLIBRI_EVENT_NONE)
                focus_window(dlg.slot);
        }
        else
        switch(gui_event)
        {
        case KOLIBRI_EVENT_NONE:
            last_clip_num = kol_clip_num();
            if (prev_clip_num == last_clip_num) break;
            prev_clip_num = last_clip_num;
            draw_window();  // recheck clipboard every 0.1s
			break;
/*
        case KOLIBRI_EVENT_REDRAW:
            draw_window();
			break;
*/
        case KOLIBRI_EVENT_KEY:
            // scroll
			break;
        case KOLIBRI_EVENT_BUTTON:
            pressed_button = get_os_button();
            switch (pressed_button)
            {
              case BTN_POP:
                kol_clip_pop();
                strcpy(statusbar, "POP pressed....");
                break;
              case BTN_UNLOCK:
                kol_clip_unlock();
                strcpy(statusbar, "UNLOCK pressed....");
                draw_window();
                break;
              case BTN_QUIT:
                return 0;
                break;
            }
            break;
        case KOLIBRI_EVENT_MOUSE:
            mouse_pos = get_mouse_pos(POS_WINDOW); // window relative
            mouse_button = get_mouse_eventstate();
            debug_board_printf("mouse ev (%d,%d)%x\n", mouse_pos.x, mouse_pos.y, mouse_button);
            if (mouse_button & (1<<24)) // double click
            {
                int n = (mouse_pos.y - 60) / FONT_H;
                if (n < 0 || n >= kol_clip_num()) break;
                debug_board_printf("click on str(%d), clip slot(%d)\n", n, kol_clip_num() - n - 1);
                show_clipslot_modal(kol_clip_num() - n - 1);
            }
            // ignore
            break;
        }
        gui_event = wait_for_event(10); // 100 = 1 sec
    } while(1) ; /* End of main activity loop */

  return 0;
}

void show_clipslot_modal(int clip_slot)
{
    static char* message_text;
    char *clip_item = kol_clip_get(clip_slot);
    if((int)clip_item == 1 || (int)clip_item == -1) return;

    while(1)
    {
        uint32_t nbytes, fmt, extra = 0;
        nbytes = *(int32_t*)clip_item;
        if(nbytes < 10) break;

        fmt = *(int32_t*)(clip_item + 4);
        if(fmt == 0) // TEXT
        {
            extra = *(int32_t*)(clip_item + 8);  // encoding 0 = UTF, 1 = 0866, 2 = 1251
            message_text = realloc(message_text, nbytes);  // garantee than no more
            strncpy(message_text, clip_item + 12, nbytes-12); message_text[nbytes-12] = '\0';
            dlg.param = message_text;
            debug_board_printf("start dialog[%X](%X)\n", &dlg, dlg.param);
            child_window_create(&dlg);
        }
        else break;

        break;
    }
    user_free(clip_item);
}


void __attribute__ ((noinline)) debug_board_write_str(const char* str){
  while(*str)
    debug_board_write_byte(*str++);
}

void __attribute__ ((noinline)) debug_board_printf(const char *format,...)
{
        va_list ap;
        char log_board[300];

        va_start (ap, format);
        tiny_vsnprintf(log_board, sizeof log_board, format, ap);
        va_end(ap);
        debug_board_write_str(log_board);

}
